@@ -49,6 +49,7 @@
 #include "ck-connector.h"
 
 #include "gdm-common.h"
+#include "gdm-log.h"
 #include "gdm-session-worker.h"
 #include "gdm-marshal.h"
 
@@ -92,7 +93,7 @@ enum {
         GDM_SESSION_WORKER_STATE_AUTHENTICATED,
         GDM_SESSION_WORKER_STATE_AUTHORIZED,
         GDM_SESSION_WORKER_STATE_ACCREDITED,
-        GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED,
+        GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED,
         GDM_SESSION_WORKER_STATE_SESSION_OPENED,
         GDM_SESSION_WORKER_STATE_SESSION_STARTED,
         GDM_SESSION_WORKER_STATE_REAUTHENTICATED,
@@ -591,6 +592,7 @@ static void
 attempt_to_load_user_settings (GdmSessionWorker *worker,
                                const char       *username)
 {
+        g_debug ("GdmSessionWorker: attempting to load user settings");
         gdm_session_settings_load (worker->priv->user_settings,
                                    username);
 }
@@ -1060,8 +1062,8 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                  */
                 g_set_error (error,
                              GDM_SESSION_WORKER_ERROR,
-                             GDM_SESSION_WORKER_ERROR_AUTHENTICATING,
-                             _("error initiating conversation with authentication system: %s"),
+                             GDM_SESSION_WORKER_ERROR_SERVICE_UNAVAILABLE,
+                             _("error initiating conversation with authentication system - %s"),
                              error_code == PAM_ABORT? _("general failure") :
                              error_code == PAM_BUF_ERR? _("out of memory") :
                              error_code == PAM_SYSTEM_ERR? _("application programmer error") :
@@ -1172,7 +1174,15 @@ gdm_session_worker_authenticate_user (GdmSessionWorker *worker,
         /* blocking call, does the actual conversation */
         error_code = pam_authenticate (worker->priv->pam_handle, authentication_flags);
 
-        if (error_code != PAM_SUCCESS) {
+        if (error_code == PAM_AUTHINFO_UNAVAIL) {
+                g_debug ("GdmSessionWorker: authentication service unavailable");
+
+                g_set_error (error,
+                             GDM_SESSION_WORKER_ERROR,
+                             GDM_SESSION_WORKER_ERROR_SERVICE_UNAVAILABLE,
+                             "%s", pam_strerror (worker->priv->pam_handle, error_code));
+                goto out;
+        } else if (error_code != PAM_SUCCESS) {
                 g_debug ("GdmSessionWorker: authentication returned %d: %s", error_code, pam_strerror (worker->priv->pam_handle, error_code));
 
                 /*
@@ -1734,10 +1744,7 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
                 dup2 (fd, STDERR_FILENO);
                 close (fd);
 
-                if (!gdm_session_settings_save (worker->priv->user_settings,
-                                                worker->priv->username)) {
-                        g_warning ("could not save session and language settings");
-                }
+                gdm_log_shutdown ();
 
                 /*
                  * Reset SIGPIPE to default so that any process in the user
@@ -1751,6 +1758,7 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
                                      environment,
                                      TRUE);
 
+                gdm_log_init ();
                 g_debug ("GdmSessionWorker: child '%s' could not be started: %s",
                          worker->priv->arguments[0],
                          g_strerror (errno));
@@ -1783,7 +1791,7 @@ gdm_session_worker_open_user_session (GdmSessionWorker  *worker,
 {
         int error_code;
 
-        g_assert (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED);
+        g_assert (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED);
         g_assert (geteuid () == 0);
 
         error_code = pam_open_session (worker->priv->pam_handle, 0);
@@ -1948,6 +1956,8 @@ on_saved_language_name_read (GdmSessionWorker *worker)
         char *language_name;
 
         language_name = gdm_session_settings_get_language_name (worker->priv->user_settings);
+
+        g_debug ("GdmSessionWorker: Saved language is %s", language_name);
         send_dbus_string_method (worker->priv->connection,
                                  "SavedLanguageNameRead",
                                  language_name);
@@ -1960,6 +1970,8 @@ on_saved_session_name_read (GdmSessionWorker *worker)
         char *session_name;
 
         session_name = gdm_session_settings_get_session_name (worker->priv->user_settings);
+
+        g_debug ("GdmSessionWorker: Saved session is %s", session_name);
         send_dbus_string_method (worker->priv->connection,
                                  "SavedSessionNameRead",
                                  session_name);
@@ -1972,33 +1984,6 @@ do_setup (GdmSessionWorker *worker)
         GError  *error;
         gboolean res;
 
-        worker->priv->user_settings = gdm_session_settings_new ();
-
-        g_signal_connect_swapped (worker->priv->user_settings,
-                                  "notify::language-name",
-                                  G_CALLBACK (on_saved_language_name_read),
-                                  worker);
-
-        g_signal_connect_swapped (worker->priv->user_settings,
-                                  "notify::session-name",
-                                  G_CALLBACK (on_saved_session_name_read),
-                                  worker);
-
-        /* In some setups the user can read ~/.dmrc at this point.
-         * In some other setups the user can only read ~/.dmrc after completing
-         * the pam conversation.
-         *
-         * The user experience is better if we can read .dmrc now since we can
-         * prefill in the language and session combo boxes in the greeter with
-         * the right values.
-         *
-         * We'll try now, and if it doesn't work out, try later.
-         */
-        if (worker->priv->username != NULL) {
-                attempt_to_load_user_settings (worker,
-                                               worker->priv->username);
-        }
-
         error = NULL;
         res = gdm_session_worker_initialize_pam (worker,
                                                  worker->priv->service,
@@ -2094,6 +2079,20 @@ do_accredit (GdmSessionWorker *worker)
         send_dbus_void_method (worker->priv->connection, "Accredited");
 }
 
+static void
+save_account_details_now (GdmSessionWorker *worker)
+{
+        g_assert (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCREDITED);
+
+        g_debug ("GdmSessionWorker: saving account details for user %s", worker->priv->username);
+        worker->priv->state = GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED;
+        if (!gdm_session_settings_save (worker->priv->user_settings,
+                                        worker->priv->username)) {
+                g_warning ("could not save session and language settings");
+        }
+        queue_state_change (worker);
+}
+
 static void
 on_settings_is_loaded_changed (GdmSessionWorker *worker)
 {
@@ -2101,9 +2100,14 @@ on_settings_is_loaded_changed (GdmSessionWorker *worker)
                 return;
         }
 
-        if (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCREDITED) {
-                worker->priv->state = GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED;
+        if (worker->priv->state == GDM_SESSION_WORKER_STATE_NONE) {
+                g_debug ("GdmSessionWorker: queuing setup for user: %s %s",
+                         worker->priv->username, worker->priv->display_device);
                 queue_state_change (worker);
+        } else if (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCREDITED) {
+                save_account_details_now (worker);
+        } else {
+                return;
         }
 
         g_signal_handlers_disconnect_by_func (G_OBJECT (worker->priv->user_settings),
@@ -2112,23 +2116,23 @@ on_settings_is_loaded_changed (GdmSessionWorker *worker)
 }
 
 static void
-do_fetch_account_details (GdmSessionWorker *worker)
+do_save_account_details_when_ready (GdmSessionWorker *worker)
 {
         g_assert (worker->priv->state == GDM_SESSION_WORKER_STATE_ACCREDITED);
 
-        if (gdm_session_settings_is_loaded (worker->priv->user_settings)) {
-                worker->priv->state = GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED;
-                queue_state_change (worker);
+        if (!gdm_session_settings_is_loaded (worker->priv->user_settings)) {
+                g_signal_connect (G_OBJECT (worker->priv->user_settings),
+                                  "notify::is-loaded",
+                                  G_CALLBACK (on_settings_is_loaded_changed),
+                                  worker);
+                g_debug ("GdmSessionWorker: user %s, not fully loaded yet, will save account details later",
+                         worker->priv->username);
+                gdm_session_settings_load (worker->priv->user_settings,
+                                           worker->priv->username);
                 return;
         }
 
-        g_signal_connect (G_OBJECT (worker->priv->user_settings),
-                          "notify::is-loaded",
-                          G_CALLBACK (on_settings_is_loaded_changed),
-                          worker);
-
-        gdm_session_settings_load (worker->priv->user_settings,
-                                   worker->priv->username);
+        save_account_details_now (worker);
 }
 
 static void
@@ -2194,8 +2198,8 @@ get_state_name (int state)
         case GDM_SESSION_WORKER_STATE_ACCREDITED:
                 name = "ACCREDITED";
                 break;
-        case GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED:
-                name = "ACCOUNT_DETAILS_FETCHED";
+        case GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED:
+                name = "ACCOUNT_DETAILS_SAVED";
                 break;
         case GDM_SESSION_WORKER_STATE_SESSION_OPENED:
                 name = "SESSION_OPENED";
@@ -2235,8 +2239,8 @@ state_change_idle (GdmSessionWorker *worker)
         case GDM_SESSION_WORKER_STATE_ACCREDITED:
                 do_accredit (worker);
                 break;
-        case GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_FETCHED:
-                do_fetch_account_details (worker);
+        case GDM_SESSION_WORKER_STATE_ACCOUNT_DETAILS_SAVED:
+                do_save_account_details_when_ready (worker);
                 break;
         case GDM_SESSION_WORKER_STATE_SESSION_OPENED:
                 do_open_session (worker);
@@ -2381,8 +2385,27 @@ on_setup_for_user (GdmSessionWorker *worker,
                 worker->priv->hostname = g_strdup (hostname);
                 worker->priv->username = g_strdup (username);
 
-                g_debug ("GdmSessionWorker: queuing setup for user: %s %s", service, console);
-                queue_state_change (worker);
+                g_signal_connect_swapped (worker->priv->user_settings,
+                                          "notify::language-name",
+                                          G_CALLBACK (on_saved_language_name_read),
+                                          worker);
+
+                g_signal_connect_swapped (worker->priv->user_settings,
+                                          "notify::session-name",
+                                          G_CALLBACK (on_saved_session_name_read),
+                                          worker);
+
+                /* Load settings from accounts daemon before continuing
+                 * FIXME: need to handle username not loading
+                 */
+                if (gdm_session_settings_load (worker->priv->user_settings, username)) {
+                        queue_state_change (worker);
+                } else {
+                        g_signal_connect (G_OBJECT (worker->priv->user_settings),
+                                          "notify::is-loaded",
+                                          G_CALLBACK (on_settings_is_loaded_changed),
+                                          worker);
+                }
         } else {
                 g_warning ("Unable to get arguments: %s", error.message);
                 dbus_error_free (&error);
@@ -2631,6 +2654,8 @@ gdm_session_worker_init (GdmSessionWorker *worker)
 {
 
         worker->priv = GDM_SESSION_WORKER_GET_PRIVATE (worker);
+
+        worker->priv->user_settings = gdm_session_settings_new ();
 }
 
 static void
@@ -2658,6 +2683,7 @@ gdm_session_worker_finalize (GObject *object)
 
         gdm_session_worker_unwatch_child (worker);
 
+        g_object_unref (worker->priv->user_settings);
         g_free (worker->priv->service);
         g_free (worker->priv->x11_display_name);
         g_free (worker->priv->x11_authority_file);
